from spacyjsonnlp import SpacyPipeline
from pyjsonnlp.microservices.flask_server import FlaskMicroservice

app = FlaskMicroservice(__name__, SpacyPipeline(), base_route='/')
app.with_constituents = True
app.with_coreferences = True
app.with_dependencies = True
app.with_expressions = True

if __name__ == "__main__":
    app.run(debug=True, port=5001)



from collections import OrderedDict
from unittest import TestCase

from spacyjsonnlp.dependencies import DependencyAnnotator

j = OrderedDict({
  "meta": {
    "DC.conformsTo": "0.2.9",
    "DC.created": "2019-04-25T20:31:28",
    "DC.date": "2019-04-25T20:31:28"
  },
  "documents": {
    1: {
      "meta": {
        "DC.conformsTo": "0.2.9",
        "DC.source": "SpaCy 2.1.3",
        "DC.created": "2019-04-25T20:31:28",
        "DC.date": "2019-04-25T20:31:28",
        "DC.language": "en"
      },
      "id": 1,
      "text": "I want to buy a big red car.",
      "tokenList": OrderedDict({
        1: {
          "id": 1,
          "text": "I",
          "lemma": "-PRON-",
          "xpos": "PRP",
          "upos": "PRON",
          "entity_iob": "O",
          "characterOffsetBegin": 0,
          "characterOffsetEnd": 1,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "Yes",
            "Alpha": "Yes",
            "PronType": "Prs",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "Yes"
          },
          "shape": "X"
        },
        2: {
          "id": 2,
          "text": "want",
          "lemma": "want",
          "xpos": "VBP",
          "upos": "VERB",
          "entity_iob": "O",
          "characterOffsetBegin": 2,
          "characterOffsetEnd": 6,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "No",
            "Alpha": "Yes",
            "VerbForm": "Fin",
            "Tense": "Pres",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "Yes"
          },
          "shape": "xxxx"
        },
        3: {
          "id": 3,
          "text": "to",
          "lemma": "to",
          "xpos": "TO",
          "upos": "PART",
          "entity_iob": "O",
          "characterOffsetBegin": 7,
          "characterOffsetEnd": 9,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "Yes",
            "Alpha": "Yes",
            "PartType": "Inf",
            "VerbForm": "Inf",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "Yes"
          },
          "shape": "xx"
        },
        4: {
          "id": 4,
          "text": "buy",
          "lemma": "buy",
          "xpos": "VB",
          "upos": "VERB",
          "entity_iob": "O",
          "characterOffsetBegin": 10,
          "characterOffsetEnd": 13,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "No",
            "Alpha": "Yes",
            "VerbForm": "Inf",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "Yes"
          },
          "shape": "xxx"
        },
        5: {
          "id": 5,
          "text": "a",
          "lemma": "a",
          "xpos": "DT",
          "upos": "DET",
          "entity_iob": "O",
          "characterOffsetBegin": 14,
          "characterOffsetEnd": 15,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "Yes",
            "Alpha": "Yes",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "Yes"
          },
          "shape": "x"
        },
        6: {
          "id": 6,
          "text": "big",
          "lemma": "big",
          "xpos": "JJ",
          "upos": "ADJ",
          "entity_iob": "O",
          "characterOffsetBegin": 16,
          "characterOffsetEnd": 19,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "No",
            "Alpha": "Yes",
            "Degree": "Pos",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "Yes"
          },
          "shape": "xxx"
        },
        7: {
          "id": 7,
          "text": "red",
          "lemma": "red",
          "xpos": "JJ",
          "upos": "ADJ",
          "entity_iob": "O",
          "characterOffsetBegin": 20,
          "characterOffsetEnd": 23,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "No",
            "Alpha": "Yes",
            "Degree": "Pos",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "Yes"
          },
          "shape": "xxx"
        },
        8: {
          "id": 8,
          "text": "car",
          "lemma": "car",
          "xpos": "NN",
          "upos": "NOUN",
          "entity_iob": "O",
          "characterOffsetBegin": 24,
          "characterOffsetEnd": 27,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "No",
            "Alpha": "Yes",
            "Number": "Sing",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "No"
          },
          "shape": "xxx"
        },
        9: {
          "id": 9,
          "text": ".",
          "lemma": ".",
          "xpos": ".",
          "upos": "PUNCT",
          "entity_iob": "O",
          "characterOffsetBegin": 27,
          "characterOffsetEnd": 28,
          "lang": "en",
          "features": {
            "Overt": "Yes",
            "Stop": "No",
            "Alpha": "No",
            "PunctType": "Peri",
            "Foreign": "No"
          },
          "misc": {
            "SpaceAfter": "No"
          }
        }
      }),
      "sentences": {
        1: {
          "id": 1,
          "tokenFrom": 1,
          "tokenTo": 10,
          "tokens": [
            1,
            2,
            3,
            4,
            5,
            6,
            7,
            8,
            9
          ]
        }
      },
      "dependencies": [
        {
          "style": "universal",
          "arcs": {
            1: [
              {
                "sentenceId": 1,
                "label": "nsubj",
                "governor": 2,
                "dependent": 1
              }
            ],
            2: [
              {
                "sentenceId": 1,
                "label": "root",
                "governor": 0,
                "dependent": 2
              }
            ],
            3: [
              {
                "sentenceId": 1,
                "label": "aux",
                "governor": 4,
                "dependent": 3
              }
            ],
            4: [
              {
                "sentenceId": 1,
                "label": "xcomp",
                "governor": 2,
                "dependent": 4
              }
            ],
            5: [
              {
                "sentenceId": 1,
                "label": "det",
                "governor": 8,
                "dependent": 5
              }
            ],
            6: [
              {
                "sentenceId": 1,
                "label": "amod",
                "governor": 8,
                "dependent": 6
              }
            ],
            7: [
              {
                "sentenceId": 1,
                "label": "amod",
                "governor": 8,
                "dependent": 7
              }
            ],
            8: [
              {
                "sentenceId": 1,
                "label": "dobj",
                "governor": 4,
                "dependent": 8
              }
            ],
            9: [
              {
                "sentenceId": 1,
                "label": "punct",
                "governor": 2,
                "dependent": 9
              }
            ]
          }
        }
      ],
      "constituents": [
        {
          "sentenceId": 1,
          "labeledBracketing": "(ROOT (S (NP (PRP I)) (VP (VBP want) (S (VP (TO to) (VP (VB buy) (NP (DT a) (JJ big) (JJ red) (NN car)))))) (. .)))"
        }
      ],
      "expressions": [
        {
          "id": 1,
          "type": "NP",
          "head": 8,
          "dependency": "dobj",
          "tokens": [
            5,
            6,
            7,
            8
          ]
        }
      ]
    }
  }
})


class TestDependencyAnnotator(TestCase):
    def setUp(self) -> None:
        self.d = DependencyAnnotator()

    def test_annotate(self):
        nlp_json = OrderedDict(j)
        self.d.annotate(nlp_json)
        sent = nlp_json['documents'][1]['sentences'][1]
        assert [2] == sent['root'], sent['root']
        expected = {'head': 2, 'semantic': [2], 'phrase': [1, 2, 3, 4, 5, 6, 7, 8, 9]}
        assert expected == sent['mainVerb'], sent['mainVerb']
        expected = {'head': 1, 'semantic': [1], 'phrase': [1]}
        assert expected == sent['subject'], sent['subject']
        assert not sent['compound']
        assert sent['complex']
        assert not sent['negated']

        expected = {1: {'id': 1, 'sentenceId': 1, 'clauseType': 'relative', 'tokens': [3, 4, 5, 6, 7, 8], 'root': [4], 'mainVerb': {'head': 4, 'semantic': [4], 'phrase': [3, 4, 5, 6, 7, 8]}, 'object': {'head': 8, 'semantic': [8], 'phrase': [5, 6, 7, 8]}, 'compound': False, 'complex': False, 'transitivity': 'transitive', 'negated': False, 'parentClauseId': 2}, 2: {'id': 2, 'sentenceId': 1, 'clauseType': 'matrix', 'tokens': [1, 2, 9], 'root': [2], 'mainVerb': {'head': 2, 'semantic': [2], 'phrase': [1, 2, 3, 4, 5, 6, 7, 8, 9]}, 'subject': {'head': 1, 'semantic': [1], 'phrase': [1]}, 'compound': False, 'complex': True, 'negated': False}}
        actual = nlp_json['documents'][1]['clauses']
        assert expected == actual, actual



from collections import OrderedDict
from unittest import TestCase

import pyjsonnlp
import pytest
from pyjsonnlp import validation

from spacyjsonnlp import SpacyPipeline, get_model
from . import mocks

text = "Autonomous cars from the countryside of France shift insurance liability toward manufacturers. People are afraid that they will crash."


class TestSpacy(TestCase):
    def test_process(self):
        pyjsonnlp.__version__ = '0.1'
        actual = SpacyPipeline.process(text, spacy_model='en', coreferences=False, constituents=False)
        expected = OrderedDict([('meta', OrderedDict([('DC.conformsTo', '0.1'), ('DC.created', '2019-01-25T17:04:34'), ('DC.date', '2019-01-25T17:04:34')])), ('documents', {1: OrderedDict([('meta', OrderedDict([('DC.conformsTo', '0.1'), ('DC.source', 'SpaCy 2.1.3'), ('DC.created', '2019-01-25T17:04:34'), ('DC.date', '2019-01-25T17:04:34'), ('DC.language', 'en')])), ('id', 1), ('text', 'Autonomous cars from the countryside of France shift insurance liability toward manufacturers. People are afraid that they will crash.'), ('tokenList', {1: {'id': 1, 'text': 'Autonomous', 'lemma': 'autonomous', 'xpos': 'JJ', 'upos': 'ADJ', 'entity_iob': 'O', 'characterOffsetBegin': 0, 'characterOffsetEnd': 10, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Degree': 'Pos', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx'}, 2: {'id': 2, 'text': 'cars', 'lemma': 'car', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 11, 'characterOffsetEnd': 15, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 3: {'id': 3, 'text': 'from', 'lemma': 'from', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 16, 'characterOffsetEnd': 20, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 4: {'id': 4, 'text': 'the', 'lemma': 'the', 'xpos': 'DT', 'upos': 'DET', 'entity_iob': 'O', 'characterOffsetBegin': 21, 'characterOffsetEnd': 24, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxx'}, 5: {'id': 5, 'text': 'countryside', 'lemma': 'countryside', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 25, 'characterOffsetEnd': 36, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 6: {'id': 6, 'text': 'of', 'lemma': 'of', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 37, 'characterOffsetEnd': 39, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xx'}, 7: {'id': 7, 'text': 'France', 'lemma': 'France', 'xpos': 'NNP', 'upos': 'PROPN', 'entity_iob': 'B', 'characterOffsetBegin': 40, 'characterOffsetEnd': 46, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'NounType': 'Prop', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx', 'entity': 'GPE'}, 8: {'id': 8, 'text': 'shift', 'lemma': 'shift', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 47, 'characterOffsetEnd': 52, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 9: {'id': 9, 'text': 'insurance', 'lemma': 'insurance', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 53, 'characterOffsetEnd': 62, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 10: {'id': 10, 'text': 'liability', 'lemma': 'liability', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 63, 'characterOffsetEnd': 72, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 11: {'id': 11, 'text': 'toward', 'lemma': 'toward', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 73, 'characterOffsetEnd': 79, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 12: {'id': 12, 'text': 'manufacturers', 'lemma': 'manufacturer', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 80, 'characterOffsetEnd': 93, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}, 'shape': 'xxxx'}, 13: {'id': 13, 'text': '.', 'lemma': '.', 'xpos': '.', 'upos': 'PUNCT', 'entity_iob': 'O', 'characterOffsetBegin': 93, 'characterOffsetEnd': 94, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'No', 'PunctType': 'Peri', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}}, 14: {'id': 14, 'text': 'People', 'lemma': 'People', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 94, 'characterOffsetEnd': 100, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx'}, 15: {'id': 15, 'text': 'are', 'lemma': 'be', 'xpos': 'VBP', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 101, 'characterOffsetEnd': 104, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'VerbForm': 'Fin', 'Tense': 'Pres', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxx'}, 16: {'id': 16, 'text': 'afraid', 'lemma': 'afraid', 'xpos': 'JJ', 'upos': 'ADJ', 'entity_iob': 'O', 'characterOffsetBegin': 105, 'characterOffsetEnd': 111, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Degree': 'Pos', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 17: {'id': 17, 'text': 'that', 'lemma': 'that', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 112, 'characterOffsetEnd': 116, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 18: {'id': 18, 'text': 'they', 'lemma': '-PRON-', 'xpos': 'PRP', 'upos': 'PRON', 'entity_iob': 'O', 'characterOffsetBegin': 117, 'characterOffsetEnd': 121, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'PronType': 'Prs', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 19: {'id': 19, 'text': 'will', 'lemma': 'will', 'xpos': 'MD', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 122, 'characterOffsetEnd': 126, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'VerbType': 'Mod', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 20: {'id': 20, 'text': 'crash', 'lemma': 'crash', 'xpos': 'VB', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 127, 'characterOffsetEnd': 132, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'VerbForm': 'Inf', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}, 'shape': 'xxxx'}, 21: {'id': 21, 'text': '.', 'lemma': '.', 'xpos': '.', 'upos': 'PUNCT', 'entity_iob': 'O', 'characterOffsetBegin': 132, 'characterOffsetEnd': 133, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'No', 'PunctType': 'Peri', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}}}), ('clauses', {1: {'id': 1, 'sentenceId': 2, 'clauseType': 'complement', 'tokens': [17, 18, 19, 20], 'root': [20], 'mainVerb': {'head': 20, 'semantic': [20], 'phrase': [17, 18, 19, 20]}, 'subject': {'head': 18, 'semantic': [18], 'phrase': [18]}, 'compound': False, 'complex': False, 'transitivity': 'intransitive', 'negated': False, 'parentClauseId': 2}, 2: {'id': 2, 'sentenceId': 2, 'clauseType': 'matrix', 'tokens': [14, 15, 16, 21], 'root': [15], 'mainVerb': {'head': 15, 'semantic': [15], 'phrase': [14, 15, 16, 17, 18, 19, 20, 21]}, 'subject': {'head': 14, 'semantic': [14], 'phrase': [14]}, 'compound': False, 'complex': True, 'negated': False}}), ('sentences', {1: {'id': 1, 'tokenFrom': 1, 'tokenTo': 14, 'tokens': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13], 'root': [2], 'compound': False, 'complex': False, 'negated': False}, 2: {'id': 2, 'tokenFrom': 14, 'tokenTo': 22, 'tokens': [14, 15, 16, 17, 18, 19, 20, 21], 'root': [15], 'mainVerb': {'head': 15, 'semantic': [15], 'phrase': [14, 15, 16, 17, 18, 19, 20, 21]}, 'subject': {'head': 14, 'semantic': [14], 'phrase': [14]}, 'compound': False, 'complex': True, 'negated': False}}), ('dependencies', [{'style': 'universal', 'arcs': {1: [{'sentenceId': 1, 'label': 'amod', 'governor': 2, 'dependent': 1}], 2: [{'sentenceId': 1, 'label': 'root', 'governor': 0, 'dependent': 2}], 3: [{'sentenceId': 1, 'label': 'prep', 'governor': 2, 'dependent': 3}], 4: [{'sentenceId': 1, 'label': 'det', 'governor': 5, 'dependent': 4}], 5: [{'sentenceId': 1, 'label': 'pobj', 'governor': 3, 'dependent': 5}], 6: [{'sentenceId': 1, 'label': 'prep', 'governor': 5, 'dependent': 6}], 7: [{'sentenceId': 1, 'label': 'compound', 'governor': 10, 'dependent': 7}], 8: [{'sentenceId': 1, 'label': 'compound', 'governor': 9, 'dependent': 8}], 9: [{'sentenceId': 1, 'label': 'compound', 'governor': 10, 'dependent': 9}], 10: [{'sentenceId': 1, 'label': 'pobj', 'governor': 6, 'dependent': 10}], 11: [{'sentenceId': 1, 'label': 'prep', 'governor': 10, 'dependent': 11}], 12: [{'sentenceId': 1, 'label': 'pobj', 'governor': 11, 'dependent': 12}], 13: [{'sentenceId': 1, 'label': 'punct', 'governor': 2, 'dependent': 13}], 14: [{'sentenceId': 2, 'label': 'nsubj', 'governor': 15, 'dependent': 14}], 15: [{'sentenceId': 2, 'label': 'root', 'governor': 0, 'dependent': 15}], 16: [{'sentenceId': 2, 'label': 'acomp', 'governor': 15, 'dependent': 16}], 17: [{'sentenceId': 2, 'label': 'mark', 'governor': 20, 'dependent': 17}], 18: [{'sentenceId': 2, 'label': 'nsubj', 'governor': 20, 'dependent': 18}], 19: [{'sentenceId': 2, 'label': 'aux', 'governor': 20, 'dependent': 19}], 20: [{'sentenceId': 2, 'label': 'ccomp', 'governor': 16, 'dependent': 20}], 21: [{'sentenceId': 2, 'label': 'punct', 'governor': 15, 'dependent': 21}]}}]), ('expressions', [{'id': 1, 'type': 'NP', 'head': 2, 'dependency': 'root', 'tokens': [1, 2]}, {'id': 2, 'type': 'NP', 'head': 5, 'dependency': 'pobj', 'tokens': [4, 5]}, {'id': 3, 'type': 'NP', 'head': 10, 'dependency': 'pobj', 'tokens': [7, 8, 9, 10]}])])})])
        assert actual == expected, actual

    def test_process_coref(self):
        pyjsonnlp.__version__ = '0.1'
        actual = SpacyPipeline.process(text, spacy_model='en', coreferences=True, constituents=False)
        expected = OrderedDict([('meta', OrderedDict([('DC.conformsTo', '0.1'), ('DC.created', '2019-01-25T17:04:34'), ('DC.date', '2019-01-25T17:04:34')])), ('documents', {1: OrderedDict([('meta', OrderedDict([('DC.conformsTo', '0.1'), ('DC.source', 'SpaCy 2.1.3'), ('DC.created', '2019-01-25T17:04:34'), ('DC.date', '2019-01-25T17:04:34'), ('DC.language', 'en')])), ('id', 1), ('text', 'Autonomous cars from the countryside of France shift insurance liability toward manufacturers. People are afraid that they will crash.'), ('tokenList', {1: {'id': 1, 'text': 'Autonomous', 'lemma': 'autonomous', 'xpos': 'JJ', 'upos': 'ADJ', 'entity_iob': 'O', 'characterOffsetBegin': 0, 'characterOffsetEnd': 10, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Degree': 'Pos', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx'}, 2: {'id': 2, 'text': 'cars', 'lemma': 'car', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 11, 'characterOffsetEnd': 15, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 3: {'id': 3, 'text': 'from', 'lemma': 'from', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 16, 'characterOffsetEnd': 20, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 4: {'id': 4, 'text': 'the', 'lemma': 'the', 'xpos': 'DT', 'upos': 'DET', 'entity_iob': 'O', 'characterOffsetBegin': 21, 'characterOffsetEnd': 24, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxx'}, 5: {'id': 5, 'text': 'countryside', 'lemma': 'countryside', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 25, 'characterOffsetEnd': 36, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 6: {'id': 6, 'text': 'of', 'lemma': 'of', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 37, 'characterOffsetEnd': 39, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xx'}, 7: {'id': 7, 'text': 'France', 'lemma': 'France', 'xpos': 'NNP', 'upos': 'PROPN', 'entity_iob': 'B', 'characterOffsetBegin': 40, 'characterOffsetEnd': 46, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'NounType': 'Prop', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx', 'entity': 'GPE'}, 8: {'id': 8, 'text': 'shift', 'lemma': 'shift', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 47, 'characterOffsetEnd': 52, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 9: {'id': 9, 'text': 'insurance', 'lemma': 'insurance', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 53, 'characterOffsetEnd': 62, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 10: {'id': 10, 'text': 'liability', 'lemma': 'liability', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 63, 'characterOffsetEnd': 72, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 11: {'id': 11, 'text': 'toward', 'lemma': 'toward', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 73, 'characterOffsetEnd': 79, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 12: {'id': 12, 'text': 'manufacturers', 'lemma': 'manufacturer', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 80, 'characterOffsetEnd': 93, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}, 'shape': 'xxxx'}, 13: {'id': 13, 'text': '.', 'lemma': '.', 'xpos': '.', 'upos': 'PUNCT', 'entity_iob': 'O', 'characterOffsetBegin': 93, 'characterOffsetEnd': 94, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'No', 'PunctType': 'Peri', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}}, 14: {'id': 14, 'text': 'People', 'lemma': 'People', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 94, 'characterOffsetEnd': 100, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx'}, 15: {'id': 15, 'text': 'are', 'lemma': 'be', 'xpos': 'VBP', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 101, 'characterOffsetEnd': 104, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'VerbForm': 'Fin', 'Tense': 'Pres', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxx'}, 16: {'id': 16, 'text': 'afraid', 'lemma': 'afraid', 'xpos': 'JJ', 'upos': 'ADJ', 'entity_iob': 'O', 'characterOffsetBegin': 105, 'characterOffsetEnd': 111, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Degree': 'Pos', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 17: {'id': 17, 'text': 'that', 'lemma': 'that', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 112, 'characterOffsetEnd': 116, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 18: {'id': 18, 'text': 'they', 'lemma': '-PRON-', 'xpos': 'PRP', 'upos': 'PRON', 'entity_iob': 'O', 'characterOffsetBegin': 117, 'characterOffsetEnd': 121, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'PronType': 'Prs', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 19: {'id': 19, 'text': 'will', 'lemma': 'will', 'xpos': 'MD', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 122, 'characterOffsetEnd': 126, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'VerbType': 'Mod', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 20: {'id': 20, 'text': 'crash', 'lemma': 'crash', 'xpos': 'VB', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 127, 'characterOffsetEnd': 132, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'VerbForm': 'Inf', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}, 'shape': 'xxxx'}, 21: {'id': 21, 'text': '.', 'lemma': '.', 'xpos': '.', 'upos': 'PUNCT', 'entity_iob': 'O', 'characterOffsetBegin': 132, 'characterOffsetEnd': 133, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'No', 'PunctType': 'Peri', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}}}), ('clauses', {1: {'id': 1, 'sentenceId': 2, 'clauseType': 'complement', 'tokens': [17, 18, 19, 20], 'root': [20], 'mainVerb': {'head': 20, 'semantic': [20], 'phrase': [17, 18, 19, 20]}, 'subject': {'head': 18, 'semantic': [18], 'phrase': [18]}, 'compound': False, 'complex': False, 'transitivity': 'intransitive', 'negated': False, 'parentClauseId': 2}, 2: {'id': 2, 'sentenceId': 2, 'clauseType': 'matrix', 'tokens': [14, 15, 16, 21], 'root': [15], 'mainVerb': {'head': 15, 'semantic': [15], 'phrase': [14, 15, 16, 17, 18, 19, 20, 21]}, 'subject': {'head': 14, 'semantic': [14], 'phrase': [14]}, 'compound': False, 'complex': True, 'negated': False}}), ('sentences', {1: {'id': 1, 'tokenFrom': 1, 'tokenTo': 14, 'tokens': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13], 'root': [2], 'compound': False, 'complex': False, 'negated': False}, 2: {'id': 2, 'tokenFrom': 14, 'tokenTo': 22, 'tokens': [14, 15, 16, 17, 18, 19, 20, 21], 'root': [15], 'mainVerb': {'head': 15, 'semantic': [15], 'phrase': [14, 15, 16, 17, 18, 19, 20, 21]}, 'subject': {'head': 14, 'semantic': [14], 'phrase': [14]}, 'compound': False, 'complex': True, 'negated': False}}), ('dependencies', [{'style': 'universal', 'arcs': {1: [{'sentenceId': 1, 'label': 'amod', 'governor': 2, 'dependent': 1}], 2: [{'sentenceId': 1, 'label': 'root', 'governor': 0, 'dependent': 2}], 3: [{'sentenceId': 1, 'label': 'prep', 'governor': 2, 'dependent': 3}], 4: [{'sentenceId': 1, 'label': 'det', 'governor': 5, 'dependent': 4}], 5: [{'sentenceId': 1, 'label': 'pobj', 'governor': 3, 'dependent': 5}], 6: [{'sentenceId': 1, 'label': 'prep', 'governor': 5, 'dependent': 6}], 7: [{'sentenceId': 1, 'label': 'compound', 'governor': 10, 'dependent': 7}], 8: [{'sentenceId': 1, 'label': 'compound', 'governor': 9, 'dependent': 8}], 9: [{'sentenceId': 1, 'label': 'compound', 'governor': 10, 'dependent': 9}], 10: [{'sentenceId': 1, 'label': 'pobj', 'governor': 6, 'dependent': 10}], 11: [{'sentenceId': 1, 'label': 'prep', 'governor': 10, 'dependent': 11}], 12: [{'sentenceId': 1, 'label': 'pobj', 'governor': 11, 'dependent': 12}], 13: [{'sentenceId': 1, 'label': 'punct', 'governor': 2, 'dependent': 13}], 14: [{'sentenceId': 2, 'label': 'nsubj', 'governor': 15, 'dependent': 14}], 15: [{'sentenceId': 2, 'label': 'root', 'governor': 0, 'dependent': 15}], 16: [{'sentenceId': 2, 'label': 'acomp', 'governor': 15, 'dependent': 16}], 17: [{'sentenceId': 2, 'label': 'mark', 'governor': 20, 'dependent': 17}], 18: [{'sentenceId': 2, 'label': 'nsubj', 'governor': 20, 'dependent': 18}], 19: [{'sentenceId': 2, 'label': 'aux', 'governor': 20, 'dependent': 19}], 20: [{'sentenceId': 2, 'label': 'ccomp', 'governor': 16, 'dependent': 20}], 21: [{'sentenceId': 2, 'label': 'punct', 'governor': 15, 'dependent': 21}]}}]), ('coreferences', [{'id': 0, 'representative': {'tokens': [14], 'head': 14}, 'referents': [{'tokens': [18], 'head': 18}]}]), ('expressions', [{'id': 1, 'type': 'NP', 'head': 2, 'dependency': 'root', 'tokens': [1, 2]}, {'id': 2, 'type': 'NP', 'head': 5, 'dependency': 'pobj', 'tokens': [4, 5]}, {'id': 3, 'type': 'NP', 'head': 10, 'dependency': 'pobj', 'tokens': [7, 8, 9, 10]}])])})])
        assert actual == expected, actual

    def test_process_coref_single(self):
        pyjsonnlp.__version__ = '0.1'
        actual = SpacyPipeline.process("I am a cat.", spacy_model='en', coreferences=True, constituents=False)
        expected = OrderedDict([('meta', OrderedDict([('DC.conformsTo', '0.1'), ('DC.created', '2019-01-25T17:04:34'), ('DC.date', '2019-01-25T17:04:34')])), ('documents', {1: OrderedDict([('meta', OrderedDict([('DC.conformsTo', '0.1'), ('DC.source', 'SpaCy 2.1.3'), ('DC.created', '2019-01-25T17:04:34'), ('DC.date', '2019-01-25T17:04:34'), ('DC.language', 'en')])), ('id', 1), ('text', 'I am a cat.'), ('tokenList', {1: {'id': 1, 'text': 'I', 'lemma': '-PRON-', 'xpos': 'PRP', 'upos': 'PRON', 'entity_iob': 'O', 'characterOffsetBegin': 0, 'characterOffsetEnd': 1, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'PronType': 'Prs', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'X'}, 2: {'id': 2, 'text': 'am', 'lemma': 'be', 'xpos': 'VBP', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 2, 'characterOffsetEnd': 4, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'VerbForm': 'Fin', 'Tense': 'Pres', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xx'}, 3: {'id': 3, 'text': 'a', 'lemma': 'a', 'xpos': 'DT', 'upos': 'DET', 'entity_iob': 'O', 'characterOffsetBegin': 5, 'characterOffsetEnd': 6, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'x'}, 4: {'id': 4, 'text': 'cat', 'lemma': 'cat', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 7, 'characterOffsetEnd': 10, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}, 'shape': 'xxx'}, 5: {'id': 5, 'text': '.', 'lemma': '.', 'xpos': '.', 'upos': 'PUNCT', 'entity_iob': 'O', 'characterOffsetBegin': 10, 'characterOffsetEnd': 11, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'No', 'PunctType': 'Peri', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}}}), ('sentences', {1: {'id': 1, 'tokenFrom': 1, 'tokenTo': 6, 'tokens': [1, 2, 3, 4, 5], 'root': [2], 'mainVerb': {'head': 2, 'semantic': [2], 'phrase': [1, 2, 3, 4, 5]}, 'subject': {'head': 1, 'semantic': [1], 'phrase': [1]}, 'compound': False, 'complex': False, 'transitivity': 'intransitive', 'negated': False}}), ('dependencies', [{'style': 'universal', 'arcs': {1: [{'sentenceId': 1, 'label': 'nsubj', 'governor': 2, 'dependent': 1}], 2: [{'sentenceId': 1, 'label': 'root', 'governor': 0, 'dependent': 2}], 3: [{'sentenceId': 1, 'label': 'det', 'governor': 4, 'dependent': 3}], 4: [{'sentenceId': 1, 'label': 'attr', 'governor': 2, 'dependent': 4}], 5: [{'sentenceId': 1, 'label': 'punct', 'governor': 2, 'dependent': 5}]}}]), ('expressions', [{'id': 1, 'type': 'NP', 'head': 4, 'dependency': 'attr', 'tokens': [3, 4]}])])})])
        assert actual == expected, actual

    def test_constituents(self):
        pyjsonnlp.__version__ = '0.1'
        actual = SpacyPipeline.process(text, spacy_model='en', coreferences=False, constituents=True)
        expected = OrderedDict([('meta', OrderedDict([('DC.conformsTo', '0.1'), ('DC.created', '2019-01-25T17:04:34'), ('DC.date', '2019-01-25T17:04:34')])), ('documents', {1: OrderedDict([('meta', OrderedDict([('DC.conformsTo', '0.1'), ('DC.source', 'SpaCy 2.1.3'), ('DC.created', '2019-01-25T17:04:34'), ('DC.date', '2019-01-25T17:04:34'), ('DC.language', 'en')])), ('id', 1), ('text', 'Autonomous cars from the countryside of France shift insurance liability toward manufacturers. People are afraid that they will crash.'), ('tokenList', {1: {'id': 1, 'text': 'Autonomous', 'lemma': 'autonomous', 'xpos': 'JJ', 'upos': 'ADJ', 'entity_iob': 'O', 'characterOffsetBegin': 0, 'characterOffsetEnd': 10, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Degree': 'Pos', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx'}, 2: {'id': 2, 'text': 'cars', 'lemma': 'car', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 11, 'characterOffsetEnd': 15, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 3: {'id': 3, 'text': 'from', 'lemma': 'from', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 16, 'characterOffsetEnd': 20, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 4: {'id': 4, 'text': 'the', 'lemma': 'the', 'xpos': 'DT', 'upos': 'DET', 'entity_iob': 'O', 'characterOffsetBegin': 21, 'characterOffsetEnd': 24, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxx'}, 5: {'id': 5, 'text': 'countryside', 'lemma': 'countryside', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 25, 'characterOffsetEnd': 36, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 6: {'id': 6, 'text': 'of', 'lemma': 'of', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 37, 'characterOffsetEnd': 39, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xx'}, 7: {'id': 7, 'text': 'France', 'lemma': 'France', 'xpos': 'NNP', 'upos': 'PROPN', 'entity_iob': 'B', 'characterOffsetBegin': 40, 'characterOffsetEnd': 46, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'NounType': 'Prop', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx', 'entity': 'GPE'}, 8: {'id': 8, 'text': 'shift', 'lemma': 'shift', 'xpos': 'VBP', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 47, 'characterOffsetEnd': 52, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'VerbForm': 'Fin', 'Tense': 'Pres', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 9: {'id': 9, 'text': 'insurance', 'lemma': 'insurance', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 53, 'characterOffsetEnd': 62, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 10: {'id': 10, 'text': 'liability', 'lemma': 'liability', 'xpos': 'NN', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 63, 'characterOffsetEnd': 72, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Sing', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 11: {'id': 11, 'text': 'toward', 'lemma': 'toward', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 73, 'characterOffsetEnd': 79, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 12: {'id': 12, 'text': 'manufacturers', 'lemma': 'manufacturer', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 80, 'characterOffsetEnd': 93, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}, 'shape': 'xxxx'}, 13: {'id': 13, 'text': '.', 'lemma': '.', 'xpos': '.', 'upos': 'PUNCT', 'entity_iob': 'O', 'characterOffsetBegin': 93, 'characterOffsetEnd': 94, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'No', 'PunctType': 'Peri', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}}, 14: {'id': 14, 'text': 'People', 'lemma': 'People', 'xpos': 'NNS', 'upos': 'NOUN', 'entity_iob': 'O', 'characterOffsetBegin': 94, 'characterOffsetEnd': 100, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Number': 'Plur', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'Xxxxx'}, 15: {'id': 15, 'text': 'are', 'lemma': 'be', 'xpos': 'VBP', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 101, 'characterOffsetEnd': 104, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'VerbForm': 'Fin', 'Tense': 'Pres', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxx'}, 16: {'id': 16, 'text': 'afraid', 'lemma': 'afraid', 'xpos': 'JJ', 'upos': 'ADJ', 'entity_iob': 'O', 'characterOffsetBegin': 105, 'characterOffsetEnd': 111, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'Degree': 'Pos', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 17: {'id': 17, 'text': 'that', 'lemma': 'that', 'xpos': 'IN', 'upos': 'ADP', 'entity_iob': 'O', 'characterOffsetBegin': 112, 'characterOffsetEnd': 116, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 18: {'id': 18, 'text': 'they', 'lemma': '-PRON-', 'xpos': 'PRP', 'upos': 'PRON', 'entity_iob': 'O', 'characterOffsetBegin': 117, 'characterOffsetEnd': 121, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'PronType': 'Prs', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 19: {'id': 19, 'text': 'will', 'lemma': 'will', 'xpos': 'MD', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 122, 'characterOffsetEnd': 126, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'Yes', 'Alpha': 'Yes', 'VerbType': 'Mod', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'Yes'}, 'shape': 'xxxx'}, 20: {'id': 20, 'text': 'crash', 'lemma': 'crash', 'xpos': 'VB', 'upos': 'VERB', 'entity_iob': 'O', 'characterOffsetBegin': 127, 'characterOffsetEnd': 132, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'Yes', 'VerbForm': 'Inf', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}, 'shape': 'xxxx'}, 21: {'id': 21, 'text': '.', 'lemma': '.', 'xpos': '.', 'upos': 'PUNCT', 'entity_iob': 'O', 'characterOffsetBegin': 132, 'characterOffsetEnd': 133, 'lang': 'en', 'features': {'Overt': 'Yes', 'Stop': 'No', 'Alpha': 'No', 'PunctType': 'Peri', 'Foreign': 'No'}, 'misc': {'SpaceAfter': 'No'}}}), ('clauses', {1: {'id': 1, 'sentenceId': 2, 'clauseType': 'complement', 'tokens': [17, 18, 19, 20], 'root': [20], 'mainVerb': {'head': 20, 'semantic': [20], 'phrase': [17, 18, 19, 20]}, 'subject': {'head': 18, 'semantic': [18], 'phrase': [18]}, 'compound': False, 'complex': False, 'transitivity': 'intransitive', 'negated': False, 'parentClauseId': 2}, 2: {'id': 2, 'sentenceId': 2, 'clauseType': 'matrix', 'tokens': [14, 15, 16, 21], 'root': [15], 'mainVerb': {'head': 15, 'semantic': [15], 'phrase': [14, 15, 16, 17, 18, 19, 20, 21]}, 'subject': {'head': 14, 'semantic': [14], 'phrase': [14]}, 'compound': False, 'complex': True, 'negated': False}}), ('sentences', {1: {'id': 1, 'tokenFrom': 1, 'tokenTo': 14, 'tokens': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13], 'root': [2], 'compound': False, 'complex': False, 'negated': False}, 2: {'id': 2, 'tokenFrom': 14, 'tokenTo': 22, 'tokens': [14, 15, 16, 17, 18, 19, 20, 21], 'root': [15], 'mainVerb': {'head': 15, 'semantic': [15], 'phrase': [14, 15, 16, 17, 18, 19, 20, 21]}, 'subject': {'head': 14, 'semantic': [14], 'phrase': [14]}, 'compound': False, 'complex': True, 'negated': False}}), ('dependencies', [{'style': 'universal', 'arcs': {1: [{'sentenceId': 1, 'label': 'amod', 'governor': 2, 'dependent': 1}], 2: [{'sentenceId': 1, 'label': 'root', 'governor': 0, 'dependent': 2}], 3: [{'sentenceId': 1, 'label': 'prep', 'governor': 2, 'dependent': 3}], 4: [{'sentenceId': 1, 'label': 'det', 'governor': 5, 'dependent': 4}], 5: [{'sentenceId': 1, 'label': 'pobj', 'governor': 3, 'dependent': 5}], 6: [{'sentenceId': 1, 'label': 'prep', 'governor': 5, 'dependent': 6}], 7: [{'sentenceId': 1, 'label': 'compound', 'governor': 10, 'dependent': 7}], 8: [{'sentenceId': 1, 'label': 'compound', 'governor': 9, 'dependent': 8}], 9: [{'sentenceId': 1, 'label': 'compound', 'governor': 10, 'dependent': 9}], 10: [{'sentenceId': 1, 'label': 'pobj', 'governor': 6, 'dependent': 10}], 11: [{'sentenceId': 1, 'label': 'prep', 'governor': 10, 'dependent': 11}], 12: [{'sentenceId': 1, 'label': 'pobj', 'governor': 11, 'dependent': 12}], 13: [{'sentenceId': 1, 'label': 'punct', 'governor': 2, 'dependent': 13}], 14: [{'sentenceId': 2, 'label': 'nsubj', 'governor': 15, 'dependent': 14}], 15: [{'sentenceId': 2, 'label': 'root', 'governor': 0, 'dependent': 15}], 16: [{'sentenceId': 2, 'label': 'acomp', 'governor': 15, 'dependent': 16}], 17: [{'sentenceId': 2, 'label': 'mark', 'governor': 20, 'dependent': 17}], 18: [{'sentenceId': 2, 'label': 'nsubj', 'governor': 20, 'dependent': 18}], 19: [{'sentenceId': 2, 'label': 'aux', 'governor': 20, 'dependent': 19}], 20: [{'sentenceId': 2, 'label': 'ccomp', 'governor': 16, 'dependent': 20}], 21: [{'sentenceId': 2, 'label': 'punct', 'governor': 15, 'dependent': 21}]}}]), ('constituents', [{'sentenceId': 1, 'labeledBracketing': '(ROOT (S (NP (NP (JJ Autonomous) (NNS cars)) (PP (IN from) (NP (NP (DT the) (NN countryside)) (PP (IN of) (NP (NNP France)))))) (VP (VBP shift) (NP (NN insurance) (NN liability)) (PP (IN toward) (NP (NNS manufacturers)))) (. .)))'}, {'sentenceId': 2, 'labeledBracketing': '(ROOT (S (NP (NNS People)) (VP (VBP are) (ADJP (JJ afraid) (SBAR (IN that) (S (NP (PRP they)) (VP (MD will) (VP (VB crash))))))) (. .)))'}]), ('expressions', [{'id': 1, 'type': 'NP', 'head': 2, 'dependency': 'root', 'tokens': [1, 2]}, {'id': 2, 'type': 'NP', 'head': 5, 'dependency': 'pobj', 'tokens': [4, 5]}, {'id': 3, 'type': 'NP', 'head': 10, 'dependency': 'pobj', 'tokens': [7, 8, 9, 10]}])])})])
        assert actual == expected, actual

    def test_model_not_found(self):
        with pytest.raises(ModuleNotFoundError):
            get_model('martian_core', False, False)

    def test_validation(self):
        assert validation.is_valid(SpacyPipeline.process(text, spacy_model='en', coreferences=True))



from collections import OrderedDict
from typing import List

from pyjsonnlp.annotation import Annotator
from pyjsonnlp.dependencies import UniversalDependencyParse
from pyjsonnlp.tokenization import subtract_tokens


class DependencyAnnotator(Annotator):
    clause_arcs = ('csubj', 'ccomp', 'xcomp', 'advcl', 'acl')
    clause_types = (('csubj', 'subject'), ('xcomp', 'relative'), ('ccomp', 'complement'), ('advcl', 'adverbial'), ('acl', 'adjectival'))
    compound_arcs = ('conj', 'cc')
    sov = (('subject', 'nsubj', ()),
           ('object', 'obj', ()),
           ('object', 'dobj', ()),
           ('indirectObject', 'iobj', ()),
           ('indirectObject', 'dative', ()))

    def annotate(self, nlp_json: OrderedDict) -> None:
        for doc in nlp_json['documents'].values():
            c_id = 1
            d = UniversalDependencyParse(doc['dependencies'][0], doc['tokenList'])
            for s_id, sent in doc['sentences'].items():
                s_head = d.sentence_heads[s_id]

                # subject/object/verb
                self.annotate_item(d, s_head, sent)

                # clauses
                depth = 0
                item = sent
                item_head = s_head
                parent_clause_id = 0
                item_tokens = [d.tokens[t_id] for t_id in range(sent['tokenFrom'], sent['tokenTo'])]
                while item['complex']:
                    if 'clauses' not in doc:
                        doc['clauses'] = {}
                    for arc, clause_type in self.clause_types:
                        if d.is_arc_present_below(item_head, arc):
                            # clause
                            c_head, clause_tokens = d.get_leaves_by_arc(arc, head=item_head, sentence_id=s_id)
                            clause = self.build_clause(c_id, s_id, parent_clause_id, clause_type, clause_tokens)
                            doc['clauses'][c_id] = clause
                            self.annotate_item(d, c_head, clause)
                            parent_clause_id = c_id
                            c_id += 1

                            # matrix clause at the sentence level
                            if depth == 0:
                                matrix_tokens = subtract_tokens(item_tokens, clause_tokens)
                                matrix = self.build_clause(c_id, s_id, 0, 'matrix', matrix_tokens)
                                doc['clauses'][c_id] = matrix
                                clause['parentClauseId'] = c_id
                                self.annotate_item(d, s_head, matrix)
                                c_id += 1

                            depth += 1
                            item = clause
                            item_head = c_head
                            # don't need item tokens
                            break

    @staticmethod
    def build_clause(clause_id: int, sent_id: int, parent_clause_id: int, clause_type: str, tokens: List[dict]) -> dict:
        clause = {
            'id': clause_id,
            'sentenceId': sent_id,
            'clauseType': clause_type,
            'tokens': [t['id'] for t in tokens]
        }
        if parent_clause_id:
            clause['parentClauseId'] = parent_clause_id
        return clause

    @staticmethod
    def build_grammar(d: UniversalDependencyParse, head: int) -> dict:
        return {
            'head': head,
            'semantic': [t['id'] for t in d.collect_compounds(head)],
            'phrase': [t['id'] for t in d.get_leaves(head)]
        }

    def annotate_item(self, d: UniversalDependencyParse, head: int, item: dict) -> None:
        # root
        item['root'] = head

        # subject/object/verb
        if d.tokens[head]['upos'][0] == 'V' or d.tokens[head]['xpos'][0] == 'V':
            item['mainVerb'] = self.build_grammar(d, head)
        for k, arc, follow in self.sov:
            grammar_head = d.get_child_with_arc(head, arc, follow)
            if grammar_head:
                item[k] = self.build_grammar(d, grammar_head['id'])

        # compound/complex/fragment
        item['compound'] = any(map(lambda a: d.is_arc_present_below(head, a), self.compound_arcs))
        item['complex'] = any(map(lambda a: d.is_arc_present_below(head, a), self.clause_arcs))
        # todo fragment (the syntax parser will tell this)

        # transitivity
        if 'mainVerb' in item:
            if 'indirectObject' in item:
                item['transitivity'] = 'ditransitive'
            elif 'object' in item:
                item['transitivity'] = 'transitive'
            elif not item['complex'] and d.is_arc_present_below(head, 'nsubj'):
                item['transitivity'] = 'intransitive'

        # negation
        item['negated'] = bool(d.get_child_with_arc(head, 'neg'))

        # todo sentence types
        # todo tense
        # todo modality



import datetime


class MockResponse:
    @property
    def status_code(self):
        return 200

    @property
    def text(self):
        return u'<!DOCTYPE html>\n<html lang="en-US">\n\n  <head>\n    <meta charset=\'utf-8\'>\n    <meta http-equiv="X-UA-Compatible" content="IE=edge">\n    <meta name="viewport" content="width=device-width,maximum-scale=2">\n    <link rel="stylesheet" type="text/css" media="screen" href="/assets/css/style.css?v=c467a1053650c4b666f28aa667e19ec54af99702">\n\n<!-- Begin Jekyll SEO tag v2.5.0 -->\n<title>Relationship Extraction | NLP-progress</title>\n<meta name="generator" content="Jekyll v3.7.4" />\n<meta property="og:title" content="Relationship Extraction" />\n<meta property="og:locale" content="en_US" />\n<meta name="description" content="Repository to track the progress in Natural Language Processing (NLP), including the datasets and the current state-of-the-art for the most common NLP tasks." />\n<meta property="og:description" content="Repository to track the progress in Natural Language Processing (NLP), including the datasets and the current state-of-the-art for the most common NLP tasks." />\n<link rel="canonical" href="http://nlpprogress.com/english/relationship_extraction.html" />\n<meta property="og:url" content="http://nlpprogress.com/english/relationship_extraction.html" />\n<meta property="og:site_name" content="NLP-progress" />\n<script type="application/ld+json">\n{"headline":"Relationship Extraction","@type":"WebPage","url":"http://nlpprogress.com/english/relationship_extraction.html","description":"Repository to track the progress in Natural Language Processing (NLP), including the datasets and the current state-of-the-art for the most common NLP tasks.","@context":"http://schema.org"}</script>\n<!-- End Jekyll SEO tag -->\n\n  </head>\n\n  <body>\n\n    <!-- HEADER -->\n    <div id="header_wrap" class="outer">\n        <header class="inner">\n          <a id="forkme_banner" href="https://github.com/sebastianruder/NLP-progress">View on GitHub</a>\n\n          <h1 id="project_title">NLP-progress</h1>\n          <h2 id="project_tagline">Repository to track the progress in Natural Language Processing (NLP), including the datasets and the current state-of-the-art for the most common NLP tasks.</h2>\n\n          \n        </header>\n    </div>\n\n    <!-- MAIN CONTENT -->\n    <div id="main_content_wrap" class="outer">\n      <section id="main_content" class="inner">\n        <h1 id="relationship-extraction">Relationship Extraction</h1>\n\n<p>Relationship extraction is the task of extracting semantic relationships from a text. Extracted relationships usually\noccur between two or more entities of a certain type (e.g. Person, Organisation, Location) and fall into a number of\nsemantic categories (e.g. married to, employed by, lives in).</p>\n\n<h3 id="new-york-times-corpus">New York Times Corpus</h3>\n\n<p>The standard corpus for distantly supervised relationship extraction is the New York Times (NYT) corpus, published in\n<a href="http://www.riedelcastro.org//publications/papers/riedel10modeling.pdf">Riedel et al, 2010</a>.</p>\n\n<p>This contains text from the <a href="https://catalog.ldc.upenn.edu/ldc2008t19">New York Times Annotated Corpus</a> with named\nentities extracted from the text using the Stanford NER system and automatically linked to entities in the Freebase\nknowledge base. Pairs of named entities are labelled with relationship types by aligning them against facts in the\nFreebase knowledge base. (The process of using a separate database to provide label is known as ‘distant supervision’)</p>\n\n<p>Example:</p>\n<blockquote>\n  <p><strong>Elevation Partners</strong>, the $1.9 billion private equity group that was founded by <strong>Roger McNamee</strong></p>\n</blockquote>\n\n<p><code class="highlighter-rouge">(founded_by, Elevation_Partners, Roger_McNamee)</code></p>\n\n<p>Different papers have reported various metrics since the release of the dataset, making it difficult to compare systems\ndirectly. The main metrics used are either precision at N results or plots of the precision-recall. The range of recall\nhas increased over the years as systems improve, with earlier systems having very low precision at 30% recall.</p>\n\n<table>\n  <thead>\n    <tr>\n      <th>Model</th>\n      <th>P@10%</th>\n      <th>P@30%</th>\n      <th>Paper / Source</th>\n      <th>Code</th>\n    </tr>\n  </thead>\n  <tbody>\n    <tr>\n      <td>RESIDE (Vashishth et al., 2018)</td>\n      <td>73.6</td>\n      <td>59.5</td>\n      <td><a href="http://malllabiisc.github.io/publications/papers/reside_emnlp18.pdf">RESIDE: Improving Distantly-Supervised Neural Relation Extraction using Side Information</a></td>\n      <td><a href="https://github.com/malllabiisc/RESIDE">RESIDE</a></td>\n    </tr>\n    <tr>\n      <td>PCNN+ATT (Lin et al., 2016)</td>\n      <td>69.4</td>\n      <td>51.8</td>\n      <td><a href="http://www.aclweb.org/anthology/P16-1200">Neural Relation Extraction with Selective Attention over Instances</a></td>\n      <td><a href="https://github.com/thunlp/OpenNRE/">OpenNRE</a></td>\n    </tr>\n    <tr>\n      <td>MIML-RE (Surdeneau et al., 2012)</td>\n      <td>60.7+</td>\n      <td>-</td>\n      <td><a href="http://www.aclweb.org/anthology/D12-1042">Multi-instance Multi-label Learning for Relation Extraction</a></td>\n      <td><a href="https://nlp.stanford.edu/software/mimlre.shtml">Mimlre</a></td>\n    </tr>\n    <tr>\n      <td>MultiR (Hoffman et al., 2011)</td>\n      <td>60.9+</td>\n      <td>-</td>\n      <td><a href="http://www.aclweb.org/anthology/P11-1055">Knowledge-Based Weak Supervision for Information Extraction of Overlapping Relations</a></td>\n      <td><a href="http://aiweb.cs.washington.edu/ai/raphaelh/mr/">MultiR</a></td>\n    </tr>\n    <tr>\n      <td>(Mintz et al., 2009)</td>\n      <td>39.9+</td>\n      <td>-</td>\n      <td><a href="http://www.aclweb.org/anthology/P09-1113">Distant supervision for relation extraction without labeled data</a></td>\n      <td>\xa0</td>\n    </tr>\n  </tbody>\n</table>\n\n<p>(+) Obtained from results in the paper “Neural Relation Extraction with Selective Attention over Instances”</p>\n\n<h3 id="semeval-2010-task-8">SemEval-2010 Task 8</h3>\n\n<p><a href="http://www.aclweb.org/anthology/S10-1006">SemEval-2010</a> introduced ‘Task 8 - Multi-Way Classification of Semantic\nRelations Between Pairs of Nominals’. The task is, given a sentence and two tagged nominals, to predict the relation\nbetween those nominals <em>and</em> the direction of the relation. The dataset contains nine general semantic relations\ntogether with a tenth ‘OTHER’ relation.</p>\n\n<p>Example:</p>\n<blockquote>\n  <p>There were apples, <strong>pears</strong> and oranges in the <strong>bowl</strong>.</p>\n</blockquote>\n\n<p><code class="highlighter-rouge">(content-container, pears, bowl)</code></p>\n\n<p>The main evaluation metric used is macro-averaged F1, averaged across the nine proper relationships (i.e. excluding the\nOTHER relation), taking directionality of the relation into account.</p>\n\n<p>Several papers have used additional data (e.g. pre-trained word embeddings, WordNet) to improve performance. The figures\nreported here are the highest achieved by the model using any external resources.</p>\n\n<h4 id="end-to-end-models">End-to-End Models</h4>\n\n<table>\n  <thead>\n    <tr>\n      <th>Model</th>\n      <th>F1</th>\n      <th>Paper / Source</th>\n      <th>Code</th>\n    </tr>\n  </thead>\n  <tbody>\n    <tr>\n      <td><em>CNN-based Models</em></td>\n      <td>\xa0</td>\n      <td>\xa0</td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>Multi-Attention CNN (Wang et al. 2016)</td>\n      <td><strong>88.0</strong></td>\n      <td><a href="http://aclweb.org/anthology/P16-1123">Relation Classification via Multi-Level Attention CNNs</a></td>\n      <td><a href="https://github.com/lawlietAi/relation-classification-via-attention-model">lawlietAi’s Reimplementation</a></td>\n    </tr>\n    <tr>\n      <td>Attention CNN (Huang and Y Shen, 2016)</td>\n      <td>84.3<br />85.9<sup><a href="#footnote">*</a></sup></td>\n      <td><a href="http://www.aclweb.org/anthology/C16-1238">Attention-Based Convolutional Neural Network for Semantic Relation Extraction</a></td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>CR-CNN (dos Santos et al., 2015)</td>\n      <td>84.1</td>\n      <td><a href="https://www.aclweb.org/anthology/P15-1061">Classifying Relations by Ranking with Convolutional Neural Network</a></td>\n      <td><a href="https://github.com/pratapbhanu/CRCNN">pratapbhanu’s Reimplementation</a></td>\n    </tr>\n    <tr>\n      <td>CNN (Zeng et al., 2014)</td>\n      <td>82.7</td>\n      <td><a href="http://www.aclweb.org/anthology/C14-1220">Relation Classification via Convolutional Deep Neural Network</a></td>\n      <td><a href="https://github.com/roomylee/cnn-relation-extraction">roomylee’s Reimplementation</a></td>\n    </tr>\n    <tr>\n      <td><em>RNN-based Models</em></td>\n      <td>\xa0</td>\n      <td>\xa0</td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>Entity Attention Bi-LSTM (Lee and Seo, 2018)</td>\n      <td><strong>85.2</strong></td>\n      <td><a href="">Semantic Relation Classification via Bidirectional LSTM Networks with Entity-aware Attention using Latent Entity Typing</a></td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>Hierarchical Attention Bi-LSTM (Xiao and C Liu, 2016)</td>\n      <td>84.3</td>\n      <td><a href="http://www.aclweb.org/anthology/C16-1119">Semantic Relation Classification via Hierarchical Recurrent Neural Network with Attention</a></td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>Attention Bi-LSTM (Zhou et al., 2016)</td>\n      <td>84.0</td>\n      <td><a href="http://www.aclweb.org/anthology/P16-2034">Attention-Based Bidirectional Long Short-Term Memory Networks for Relation Classification</a></td>\n      <td><a href="https://github.com/SeoSangwoo/Attention-Based-BiLSTM-relation-extraction">SeoSangwoo’s Reimplementation</a></td>\n    </tr>\n    <tr>\n      <td>Bi-LSTM (Zhang et al., 2015)</td>\n      <td>82.7<br />84.3<sup><a href="#footnote">*</a></sup></td>\n      <td><a href="http://www.aclweb.org/anthology/Y15-1009">Bidirectional long short-term memory networks for relation classification</a></td>\n      <td>\xa0</td>\n    </tr>\n  </tbody>\n</table>\n\n<p><a name="footnote">*</a>: It uses external lexical resources, such as WordNet, part-of-speech tags, dependency tags, and named entity tags.</p>\n\n<h4 id="dependency-models">Dependency Models</h4>\n\n<table>\n  <thead>\n    <tr>\n      <th>Model</th>\n      <th>F1</th>\n      <th>Paper / Source</th>\n      <th>Code</th>\n    </tr>\n  </thead>\n  <tbody>\n    <tr>\n      <td>BRCNN (Cai et al., 2016)</td>\n      <td><strong>86.3</strong></td>\n      <td><a href="http://www.aclweb.org/anthology/P16-1072">Bidirectional Recurrent Convolutional Neural Network for Relation Classification</a></td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>DRNNs (Xu et al., 2016)</td>\n      <td>86.1</td>\n      <td><a href="https://arxiv.org/abs/1601.03651">Improved Relation Classification by Deep Recurrent Neural Networks with Data Augmentation</a></td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>depLCNN + NS (Xu et al., 2015a)</td>\n      <td>85.6</td>\n      <td><a href="https://www.aclweb.org/anthology/D/D15/D15-1062.pdf">Semantic Relation Classification via Convolutional Neural Networks with Simple Negative Sampling</a></td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>SDP-LSTM (Xu et al., 2015b)</td>\n      <td>83.7</td>\n      <td><a href="https://arxiv.org/abs/1508.03720">Classifying Relations via Long Short Term Memory Networks along Shortest Dependency Path</a></td>\n      <td><a href="https://github.com/Sshanu/Relation-Classification">Sshanu’s Reimplementation</a></td>\n    </tr>\n    <tr>\n      <td>DepNN (Liu et al., 2015)</td>\n      <td>83.6</td>\n      <td><a href="http://www.aclweb.org/anthology/P15-2047">A Dependency-Based Neural Network for Relation Classification</a></td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>FCN (Yu et al., 2014)</td>\n      <td>83.0</td>\n      <td><a href="https://www.cs.cmu.edu/~mgormley/papers/yu+gormley+dredze.nipsw.2014.pdf">Factor-based compositional embedding models</a></td>\n      <td>\xa0</td>\n    </tr>\n    <tr>\n      <td>MVRNN (Socher et al., 2012)</td>\n      <td>82.4</td>\n      <td><a href="http://aclweb.org/anthology/D12-1110">Semantic Compositionality through Recursive Matrix-Vector Spaces</a></td>\n      <td><a href="https://github.com/pratapbhanu/MVRNN">pratapbhanu’s Reimplementation</a></td>\n    </tr>\n  </tbody>\n</table>\n\n<h1 id="fewrel">FewRel</h1>\n\n<p>The Few-Shot Relation Classification Dataset (FewRel) is a different setting from the previous datasets. This dataset consists of 70K sentences expressing 100 relations annotated by crowdworkers on Wikipedia corpus. The few-shot learning task follows the N-way K-shot meta learning setting. It is both the largest supervised relation classification dataset as well as the largest few-shot learning dataset till now.</p>\n\n<p>The public leaderboard is available on the <a href="http://zhuhao.me/fewrel">FewRel website</a>.</p>\n\n<p><a href="/">Go back to the README</a></p>\n\n      </section>\n    </div>\n\n    <!-- FOOTER  -->\n    <div id="footer_wrap" class="outer">\n      <footer class="inner">\n        \n        <p class="copyright">NLP-progress maintained by <a href="https://github.com/sebastianruder">sebastianruder</a></p>\n        \n        <p>Published with <a href="https://pages.github.com">GitHub Pages</a></p>\n      </footer>\n    </div>\n\n    \n  </body>\n</html>\n'


class MockArgs:
    def __init__(self, mock_format):
        self.mock_format = mock_format

    def get(self, k, v=None):
        return self.mock_format


class MockRequest:
    def __init__(self, mock_format):
        self.mock_args = MockArgs(mock_format)

    @property
    def args(self):
        return self.mock_args


class NewDate(datetime.datetime):
    @classmethod
    def now(cls, **kwargs):
        return datetime.datetime(2019, 1, 25, 17, 4, 34, 295386)


datetime.datetime = NewDate



#!/usr/bin/env python3

"""
(C) 2019 Damir Cavar, Oren Baldinger, Maanvitha Gongalla, Anurag Kumar, Murali Kammili, Boli Fang

Wrappers for spaCy to JSON-NLP output format.

Licensed under the Apache License 2.0, see the file LICENSE for more details.

Brought to you by the NLP-Lab.org (https://nlp-lab.org/)!
"""
import functools
import re
from collections import OrderedDict, defaultdict, Counter
from typing import Dict, Tuple

import neuralcoref
import spacy
from benepar.spacy_plugin import BeneparComponent
from pyjsonnlp import get_base, get_base_document, remove_empty_fields, build_constituents, find_head, build_coreference
from pyjsonnlp.pipeline import Pipeline
from pyjsonnlp.tokenization import segment
from spacy.language import Language
from spacy.tokens import Doc

from spacyjsonnlp.dependencies import DependencyAnnotator

name = "spacypyjsonnlp"
__version__ = '0.0.10'

# allowed model names
MODEL_NAMES = ('en_core_web_sm', 'en_core_web_md', 'en_core_web_lg' 'xx_ent_wiki_sm', 'de_core_news_sm', 'es_core_news_sm',
               'pt_core_news_sm', 'fr_core_news_sm', 'it_core_news_sm', 'nl_core_news_sm')
CONSTITUENTS = {'en': 'benepar_en2', 'de': 'benepar_de'}
COREF = {'en_core_web_sm', 'en_core_web_md', 'en_core_web_lg', 'xx_ent_wiki_sm'}
WORD_REGEX = re.compile(r'^[A-Za-z]+$')

__cache = defaultdict(dict)


def cache_it(func):
    """A decorator to cache function response based on params. Add it to top of function as @cache_it."""

    global __cache

    @functools.wraps(func)
    def cached(*args):
        f_name = func.__name__
        s = ''.join(map(str, args))
        if s not in __cache[f_name]:
            __cache[f_name][s] = func(*args)
        return __cache[f_name][s]
    return cached


@cache_it
def get_model(spacy_model: str, coref: bool, constituents: bool) -> Language:
    if spacy_model == 'en':
        spacy_model = 'en_core_web_sm'
    if spacy_model not in MODEL_NAMES:
        raise ModuleNotFoundError(f'No such spaCy model "{spacy_model}"')
    nlp = spacy.load(spacy_model)
    if coref and spacy_model in COREF:
        neuralcoref.add_to_pipe(nlp)
    if constituents:
        model = CONSTITUENTS.get(spacy_model[:2], "")
        if model:
            nlp.add_pipe(BeneparComponent(model))
    return nlp


class SyntokTokenizer(object):
    def __init__(self, vocab):
        self.vocab = vocab

    def __call__(self, text):
        words = []
        spaces = []
        for sent in segment(text):
            for token in sent:
                words.append(token.value)
                spaces.append(token.space_after)
        return Doc(self.vocab, words=words, spaces=spaces)


class SpacyPipeline(Pipeline):
    @staticmethod
    def process(text: str = '', spacy_model='en_core_web_sm', coreferences=False, constituents=False, dependencies=True, expressions=True) -> OrderedDict:
        """Process provided text"""
        nlp = get_model(spacy_model, coreferences, constituents)
        nlp.tokenizer = SyntokTokenizer(nlp.vocab)
        doc = nlp(text)
        j: OrderedDict = get_base()
        d: OrderedDict = get_base_document(1)
        j['documents'][d['id']] = d

        d['meta']['DC.source'] = 'SpaCy {}'.format(spacy.__version__)
        d['text'] = text

        model_lang = spacy_model[0:2]
        lang = Counter()  # track the frequency of each language
        sent_lookup: Dict[int, int] = {}  # map sentence end_char to our index
        token_lookup: Dict[Tuple[int, int], int] = {}  # map (sent_id, spacy token index) to our token index

        # tokens and sentences
        token_id = 1
        sent_num = 1
        for sent in doc.sents:
            current_sent = {
                'id': sent_num,
                'tokenFrom': token_id,
                'tokenTo': token_id + len(sent),  # begin inclusive, end exclusive
                'tokens': []
            }
            if constituents:
                try:
                    d['constituents'].append(build_constituents(sent_num, sent._.parse_string))
                except Exception:
                    pass

            sent_lookup[sent.end_char] = sent_num
            d['sentences'][current_sent['id']] = current_sent
            last_char_index = 0
            for token in sent:
                t = {
                    'id': token_id,
                    'text': token.text,
                    'lemma': token.lemma_,
                    'xpos': token.tag_,
                    'upos': token.pos_,
                    'entity_iob': token.ent_iob_,
                    'characterOffsetBegin': token.idx,
                    'characterOffsetEnd': token.idx + len(token),
                    'lang': token.lang_,
                    'features': {
                        'Overt': True,
                        'Stop': True if token.is_stop else False,
                        'Alpha': True if token.is_alpha else False,
                    },
                    'misc': {
                        'SpaceAfter': False
                    }
                }

                # shape
                if WORD_REGEX.findall(token.text):
                    t['shape'] = token.shape_

                # space after?
                if token.idx != 0 and token.idx != last_char_index:
                    # we don't know there was a space after the previous token until we see where this one
                    # starts in relation to where the last one finished
                    d['tokenList'][token_id-1]['misc']['SpaceAfter'] = True
                last_char_index = t['characterOffsetEnd']

                # morphology
                for i, kv in enumerate(nlp.vocab.morphology.tag_map.get(token.tag_, {}).items()):
                    if i > 0:  # numeric k/v pair at the beginning
                        t['features'][kv[0]] = str(kv[1]).title()

                # entities
                if token.ent_type_:
                    t['entity'] = token.ent_type_

                # maybe check if a non-model language
                if model_lang != 'xx':
                    t['features']['Foreign'] = False if model_lang == token.lang_ else True

                # bookkeeping
                lang[token.lang_] += 1
                token_lookup[(sent_num, token.i)] = token_id
                current_sent['tokens'].append(token_id)
                d['tokenList'][token_id] = t
                token_id += 1

            d['tokenList'][token_id-1]['misc']['SpaceAfter'] = True  # EOS tokens have spaces after them
            sent_num += 1

        d['tokenList'][token_id-1]['misc']['SpaceAfter'] = False  # EOD tokens do not

        # noun phrases
        if expressions:
            chunk_id = 1
            for chunk in doc.noun_chunks:
                if len(chunk) > 1:
                    sent_id = sent_lookup[chunk.sent.sent.end_char]
                    d['expressions'].append({
                        'id': chunk_id,
                        'type': 'NP',
                        'head': token_lookup[(sent_id, chunk.root.i)],
                        'dependency': chunk.root.dep_.lower(),
                        'tokens': [token_lookup[(sent_id, token.i)] for token in chunk]
                    })
                    chunk_id += 1

        # dependencies
        if dependencies:
            deps = {
                'style': 'universal',
                'arcs': {}
            }
            d['dependencies'].append(deps)
            for sent_num, sent in enumerate(doc.sents):
                for token in sent:
                    dependent = token_lookup[(sent_num+1, token.i)]
                    deps['arcs'][dependent] = [{
                        'sentenceId': sent_num+1,
                        'label': token.dep_ if token.dep_ != 'ROOT' else 'root',
                        'governor': token_lookup[(sent_num+1, token.head.i)] if token.dep_ != 'ROOT' else 0,
                        'dependent': dependent
                    }]

            # clause, grammar extractions
            clause_annotator = DependencyAnnotator()
            clause_annotator.annotate(j)

        # coref
        # noinspection PyProtectedMember
        if coreferences and doc._.coref_clusters is not None:
            # noinspection PyProtectedMember
            for cluster in doc._.coref_clusters:
                r = build_coreference(cluster.i)
                r['representative']['tokens'] = [t.i+1 for t in cluster.main]
                r['representative']['head'] = find_head(d, r['representative']['tokens'], 'universal')
                for m in cluster.mentions:
                    if m[0].i+1 in r['representative']['tokens']:
                        continue  # don't include the representative in the mention list
                    ref = {'tokens': [t.i+1 for t in m]}
                    ref['head'] = find_head(d, ref['tokens'], 'universal')
                    r['referents'].append(ref)
                d['coreferences'].append(r)

        d['meta']['DC.language'] = max(lang)

        return remove_empty_fields(j)


if __name__ == "__main__":
    test_text = "The Mueller Report is a very long report. We spent a long time analyzing it. Trump wishes we didn't, but that didn't stop the intrepid NlpLab."
    print(SpacyPipeline.process(test_text, spacy_model='en_core_web_md', coreferences=True, constituents=False))




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="spacyjsonnlp",
    version='0.0.12',
    python_requires='>=3.6',
    author="Damir Cavar, Oren Baldinger, Maanvitha Gongalla, Anurag Kumar, Murali Kammili",
    author_email="damir@cavar.me",
    description="The Python spaCy JSON-NLP package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dcavar/spaCy-JSON-NLP",
    packages=setuptools.find_packages(),
    install_requires=[
        'spacy>=2.1',
        'neuralcoref>=4.0',
        'pyjsonnlp>=0.2.12',
        'benepar[cpu]>=0.1.2',
        'cython',
        'numpy>=1.14'
    ],
    setup_requires=["cython", "numpy>=1.14", "pytest-runner"],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    test_suite="tests",
    tests_require=["pytest", "coverage"]
)
